// 8.04.2019, Arduino IDE v1.8.9, LZ2WSG, KN34PC
// Si5351 VFO CLK0, OLED 0.91" 128x32 display, S-meter
//---------------------------------------------------------------------------------------------------------
#include "src/Rotary.h"              // http://www.buxtronix.net/2011/10/rotary-encoders-done-properly.html
#include "src/si5351.h"              // https://github.com/etherkit/Si5351Arduino, v2.1.0
#include "src/SSD1306AsciiAvrI2c.h"  // https://github.com/greiman/SSD1306Ascii

#define DEFAULT_RX    3560000
#define BAND_L        3500000
#define BAND_H        3800000
#define IF_FREQ       500000
#define SW_STEP       A0             // encoder step switch
#define PIN_SM        A1             // S-meter voltage input
#define PIN_RX_TX     A2             // RX/TX input
#define ENC_INC_1     50             // encoder increment 1
#define ENC_INC_2     500            // encoder increment 2
#define OLED_I2C_ADDR 0x3C

Rotary r = Rotary(2, 3);
Si5351 si5351(0x60);                 // Si5351 I2C address
SSD1306AsciiAvrI2c oled;

uint32_t rx;
uint32_t enc_inc;
int8_t enc_dir = 0;                  // -1 DIR_CCW, 0 DIR_NONE, 1 DIR_CCW
bool ch_flag = true;
uint16_t val_sm = 0;
uint16_t val_sm_old = 1;
bool rx_tx;
bool rx_tx_old;
bool tx_new = false;
//---------------------------------------------------------------------------------------------------------
void setup() {
  pinMode(SW_STEP, INPUT_PULLUP);
  pinMode(PIN_RX_TX, INPUT_PULLUP);

  // analogReference(INTERNAL);      // ADC reference voltage 5,0V or 1,1V (S-meter max IN voltage)

  attachInterrupt(0, rotary_encoder, CHANGE);
  attachInterrupt(1, rotary_encoder, CHANGE);

  si5351.init(SI5351_CRYSTAL_LOAD_8PF, 0, 7600);    // crystal 25.000 MHz, my correction 7600 ???

  oled.begin(&Adafruit128x32, OLED_I2C_ADDR);

  rx = DEFAULT_RX;

  rx_tx = digitalRead(PIN_RX_TX);
  rx_tx_old = !rx_tx;
  show_state();

  enc_inc = ENC_INC_1;
  show_inc();
}
//---------------------------------------------------------------------------------------------------------
void loop() {
  if (ch_flag) {
    ch_flag = false;
    rx += enc_dir * enc_inc;
    enc_dir = 0;
    rx = constrain(rx, BAND_L, BAND_H);

    // si5351.set_freq((rx + IF_FREQ) * SI5351_FREQ_MULT, SI5351_CLK0);
    // si5351.set_freq((IF_FREQ - rx) * SI5351_FREQ_MULT, SI5351_CLK0);
    si5351.set_freq((rx - IF_FREQ - 700) * SI5351_FREQ_MULT, SI5351_CLK0);

    show_freq(rx);
    tx_new = true;
  }

  if (digitalRead(SW_STEP) == LOW) { // increment events
    if (button_state(SW_STEP) > 0) {
      set_inc();
      show_inc();
      do {
      }
      while (digitalRead(SW_STEP) == LOW);
    }
  }

  rx_tx = digitalRead(PIN_RX_TX);
  if (rx_tx != rx_tx_old) {
    if (tx_new) {
      si5351.set_freq(rx * SI5351_FREQ_MULT, SI5351_CLK2);
      tx_new = false;
    }
    si5351.output_enable(SI5351_CLK2, !rx_tx);
    show_state();
    rx_tx_old = rx_tx;
  }

  val_sm = analogRead(PIN_SM);
  if ((abs(val_sm - val_sm_old)) > 3) {  // if needed draw S-meter
    show_sm();
    val_sm_old = val_sm;
  }
}
//---------------------------------------------------------------------------------------------------------
void rotary_encoder() {              // rotary encoder events
  uint8_t result = r.process();

  if (result) {
    ch_flag = true;
    if (result == DIR_CW)
      enc_dir = 1;
    else
      enc_dir = -1;
  }
}
//---------------------------------------------------------------------------------------------------------
void set_inc() {
  if (enc_inc == ENC_INC_1)
    enc_inc = ENC_INC_2;             // 500 Hz
  else
    enc_inc = ENC_INC_1;             // 50 Hz
}
//---------------------------------------------------------------------------------------------------------
void show_inc() {
  oled.setCursor(109, 3);
  oled.setFont(labels);
  if (enc_inc == ENC_INC_1)
    oled.print("6");                 // 50 Hz
  else
    oled.print("9");                 // 500 Hz
}
//---------------------------------------------------------------------------------------------------------
void show_freq(uint32_t sf_rx) {
  uint16_t fr;

  oled.setFont(lcdnums14x24);
  oled.setCursor(1, 0);
  fr = sf_rx / 1000000;
  if (fr < 10)
    oled.print(':');                 // ':' is changed to ' ' in lcdnums14x24.h
  oled.print(fr);
  oled.print('.');
  fr = (sf_rx % 1000000) / 1000;
  if (fr < 100)
    oled.print('0');
  if (fr < 10)
    oled.print('0');
  oled.print(fr);
  oled.print('.');
  fr = (sf_rx % 1000) / 10;
  if (fr < 10)
    oled.print('0');
  oled.print(fr);
}
//---------------------------------------------------------------------------------------------------------
void show_state() {
  oled.setCursor(2, 3);
  oled.setFont(labels);
  if (rx_tx)
    oled.print("1");                 // "1" is "RX" in labels.h
  else
    oled.print("2");                 // "2" is "TX" in labels.h
}
//---------------------------------------------------------------------------------------------------------
void show_sm() {
  uint8_t a = 0;
  uint8_t m = 0;

  a = (val_sm + 3) / 113;            // 1024 / 9 characters for S = 1,3,5,7,8,9,+10,+20,+30
  oled.setFont(pixels);
  oled.setCursor(24, 3);
  for (m = 0; m < a; m++)
    if (m < 6)
      oled.print('7');               // '5' - hollow rectangle, 6px
    else
      oled.print('8');               // '6' - filled rectangle, 6px
  for (m = a; m < 9; m++)
    oled.print('.');                 // '.' 1px
}
//---------------------------------------------------------------------------------------------------------
uint8_t button_state(uint8_t pin_bt) {  // push button state: 0(very fast), 1(fast), 2(slow)
  uint32_t t_start;
  uint32_t t_bt = 0;
  uint8_t bt_state;

  t_start = millis();
  while (!digitalRead(pin_bt)) {
    t_bt = millis() - t_start;
    if (t_bt > 1000)
      break;
  }
  if (t_bt < 20)                     // 'nothing' for debounce
    bt_state = 0;
  else if (t_bt < 400)               // fast
    bt_state = 1;
  else                               // slow
    bt_state = 2;
  return bt_state;
}
